###############################################################################
#                                                                             #
#                  Creating Distribution Group for Office 365                 #
#                             For Primary Group                               #
#                                    V2.3                                     #
#                       (C)2006-2012 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)]: $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_errors_count
{
$S="ErrorsCount=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function welcome
{
Param($DistributionGroupName,$DistributionGroupEmail)
write-host "Creating distribution group for Office 365 $($Version)" -foregroundcolor "white"
write-host "(C) 2006-2012 KoXo Developpement" -foregroundcolor "white"
write-host "Name=$($DistributionGroupName) Email=$($DistributionGroupEmail)"
}
###############################################################################
function proxy
{
Param([string] $Proxy,$User,$Password)
if (![String]::IsNullorEmpty($Proxy))
   {
   if (![String]::IsNullorEmpty($User))
      { 
      $SecuredPassword=ConvertTo-SecureString $Password -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $User, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection through proxy $($Proxy), User=$($User)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection through proxy $($Proxy)"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
}
###############################################################################
function connect
{
Param([string] $MyURL,$MyName,$MyPassword)
$error.Clear()
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
# Creating new runspace. 
$Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
   {
    $S="Runspace Creation was successful = $($MyURL)"
	log_message "OK" $S
	# Import-PSSession $Script:RS
   } 
   else 
   {
   $global:total_errors_count++
   $S="Runspace creation was unsuccessful to $($MyURL) || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
}
###############################################################################
function close
{
$error.Clear()
$Script:RS | Remove-PSSession -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully cleared Runspace"
	log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
    $S="Clearing Runspace was unsuccessful. Error = $($error[0])"
    log_message "ERROR" $S
	}
}
###############################################################################
function group_exists
{
Param([string] $grp)
$error.Clear()
$Group=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -arg $grp {Param($id) get-distributiongroup -Identity $id}
return (![string]::IsNullOrEmpty($Group))
}
###############################################################################
function create_distribution_group
{
Param($GroupName, 
      $GroupAlias,
      $GroupDisplayName,
      $RequireSenderAuthenticationEnabled,
      $GroupEmail,
      $GroupMembers)
$error.Clear()
%{Invoke-Command -Session $Script:RS {param ($name,$alias,$email,$display) new-distributiongroup -Name $name -Alias $alias -PrimarySmtpAddress $email -DisplayName $display -MemberJoinRestriction Closed -MemberDepartRestriction Closed}-arg $GroupName,$GroupAlias,$GroupEmail,$GroupDisplayName}>$Results
if ([String]::IsNullorEmpty($error[0]))
	{
  %{Invoke-Command -Session $Script:RS {param ($name,$authentication) set-distributiongroup -Identity $name -RequireSenderAuthenticationEnabled $authentication}-arg $GroupName,$RequireSenderAuthenticationEnabled}>$Results
  $S="Successfully create distribution group ""$($GroupName)"" DisplayName=""$($GroupDisplayName)"", Email=""$($GroupEmail)"", Alias=""$($GroupAlias)"""
	log_message "OK" $S
    # Members
    $Members=$GroupMembers.Split(",")
    ForEach ($Member in $Members)
               {
               if (![String]::IsNullorEmpty($Member))
                  {                
                  $error.Clear()
                  %{Invoke-Command -Session $Script:RS {param ($name,$memb) add-distributiongroupmember -Identity $name -Member $memb}-arg $GroupName,$Member}>$Results
                  if ([String]::IsNullorEmpty($error[0]))
                     {
                     $S="Successfully add member ""$($Member)"" for ""$($GroupName)"""
                     log_message "OK" $S
                     write-host $S
                     }
                     else
                     {
                     $global:errors_count++
                     $S="Adding member ""$($Member)"" for ""$($GroupName)"". Error = $($error[0])"
                     log_message "ERROR" $S                  
                     }
                  }   
               }
	}
	else
	{
	$global:errors_count++
    $S="Creating distribution group ""$($GroupName)"" DisplayName=""$($GroupDisplayName)"", Email=""$($GroupEmail)"", Alias=""$($GroupAlias)"". Error = $($error[0])"
    log_message "ERROR" $S
    }
}
###############################################################################
#
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V2.3"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword="%OFFICE365_PASSWORD%"
#
# Proxy parameters
#
$ProxyAddress="%PROXY_ADDRESS%"
$ProxyUser="%PROXY_USER%"     
$ProxyPassword="%PROXY_PASSWORD%"
# Only users from domain can write messages
$RequireSenderAuthenticationEnabled=$#BOOL{%OFFICE365_SENDER_AUTHENTICATION%}
#
# KoXo Administrator's variables
#
$DistributionGroupName="%PRIMARY_GROUP%"
$DistributionGroupDisplayName="%PRIMARY_GROUP_DESC%"
$DistributionGroupAlias="#LOWER{#REMOVE_SPACE{%PRIMARY_GROUP%}}"
$DistributionGroupEmail="%PRIMARY_GROUP_EMAIL%"
$DistributionGroupMembers="%PRIMARY_GROUP_USERS_EMAIL%"
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
$global:errors_count=0
welcome $DistributionGroupName $DistributionGroupEmail 
log_message "Office 365" "$($DistributionGroupDisplayName) ($($DistributionGroupEmail))"
check_version
proxy $ProxyAddress $ProxyUser $ProxyPassword
connect $Office365Url $Office365LoginName $Office365LoginPassword
if ($global:total_errors_count -eq 0)
{
if ($RequireSenderAuthenticationEnabled)
   {
   write-host "Only domain's users can write messages" -foregroundcolor "magenta"
   }
   else
   {
   write-host "Everyone can write messages" -foregroundcolor "magenta"
   }
if (group_exists($DistributionGroupAlias))
   {
   $errors_count++
   $S="Distribution group ""$($DistributionGroupAlias)"" already exists"
   write-host $S -foregroundcolor "red"
   log_message "ERROR" $S
   }
   else
   {
   create_distribution_group $DistributionGroupName $DistributionGroupAlias $DistributionGroupDisplayName $RequireSenderAuthenticationEnabled $DistributionGroupEmail $DistributionGroupMembers 
   }
log_errors_count
$global:total_errors_count=$global:total_errors_count+$global:errors_count
close
}
log_total_errors_count
