###############################################################################
#                                                                             #
#                         Modify Mailbox for Office 365                       #
#                                    V2.3                                     #
#                       (C)2006-2012 KoXo Developpement                       #
#                                                                             #
###############################################################################
function log_message
{
Param($Header,$Message)	
$LogDate=Get-Date
$LogInfo="[$($LogDate)]: [$($Header)]: $($Message)"
$LogInfo | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_total_errors_count
{
$S="TotalErrorsCount=$($global:total_errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function log_errors_count
{
Param($UserID)
$S="ErrorsCount[$($UserID)]=$($global:errors_count)"
$S | Out-File -FilePath $LogFile -Encoding Default -Append -Force
}
###############################################################################
function check_version
{
$S=Get-Host
if ($S.Version.Major -lt "2")
   {
   $S="Powershell's version must be greater or equal to 2.0 !"
   log_message "Version" $S
   $global:total_errors_count++
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host
   exit
   }
}
###############################################################################
function welcome
{
write-host "Modifying user for Office 365 $($Version)" -foregroundcolor "white"
write-host "(C) 2006-2012 KoXo Developpement" -foregroundcolor "white"
}
###############################################################################
function proxy
{
Param([string] $Proxy,$User,$Password)
if (![String]::IsNullorEmpty($Proxy))
   {
   if (![String]::IsNullorEmpty($User))
      { 
      $SecuredPassword=ConvertTo-SecureString $Password -AsPlainText -Force
      $ProxyCred=New-Object System.Management.Automation.PSCredential $User, $SecuredPassword
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate -ProxyCredential $ProxyCred
      $S="Connection through proxy $($Proxy), User=$($User)"
	  log_message "OK" $S
      }
      else 
      {
      $PSSessionOption=New-PSSessionOption -ProxyAccessType IEConfig -ProxyAuthentication Negotiate
      $S="Connection through proxy $($Proxy)"
	  log_message "OK" $S
      }
   write-host $S -foregroundcolor "cyan"   
   }
}
###############################################################################
function connect
{
Param([string] $MyURL,$MyName,$MyPassword)
$error.Clear()
# Set credential
if ( ([String]::IsNullorEmpty($MyPassword)) -or ([String]::IsNullorEmpty($MyName))  )
   {
   $Cred=Get-Credential
   }
   else
   {
   $SecuredPassword=ConvertTo-SecureString $MyPassword -AsPlainText -Force
   $Cred=New-Object System.Management.Automation.PSCredential $MyName, $SecuredPassword
   }
# Creating new runspace. 
$Script:RS=New-PSSession -ConfigurationName microsoft.exchange -ConnectionUri $MyURL -Credential $Cred -Authentication Basic -AllowRedirection -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
   {
    $S="Runspace Creation was successful = $($MyURL)"
	log_message "OK" $S
	# Import-PSSession $Script:RS
   } 
   else 
   {
   $global:total_errors_count++
   $S="Runspace creation was unsuccessful to $($MyURL) || $($error[0])"
   log_message "ERROR" $S
   # Display error
   write-host $S -foregroundcolor "red"
   write-host "Press 'Enter' to continue ..."
   Read-Host   
   }   
}
###############################################################################
function close
{
$error.Clear()
$Script:RS | Remove-PSSession -ErrorAction SilentlyContinue
if ([String]::IsNullorEmpty($error[0]))
	{
    $S="Successfully cleared Runspace"
	log_message "OK" $S
	}
	else
	{
	$global:total_errors_count++
    $S="Clearing Runspace was unsuccessful. Error = $($error[0])"
    log_message "ERROR" $S
	}
}
###############################################################################
function group_exists
{
Param([string] $grp)
$error.Clear()
$Group=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -arg $grp {Param($id) get-distributiongroup -Identity $id}
return (![string]::IsNullOrEmpty($Group))
}
###############################################################################
function user_exists
{
Param([string] $identity)
$error.Clear()
$User=Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue -arg $identity {Param($id) get-user -Identity $id}
return (![string]::IsNullOrEmpty($User))
}
###############################################################################
function modify_user
{
Param($UserID,
      $UserEmail,
      $UserDisplayName,
      $UserTitle,$UserFirstName,$UserLastName,
      $UserInitials,
      $UserPhone,$UserMobilePhone,$UserFaxNumber,
      $UserWebPage,
      $UserDepartment,$UserOffice,$UserCompany,
      $UserEmail2,$UserEmail3,$UserEmail4,$UserEmail5,
      $UserNotes,
      $MailboxPlan,
      $MemberOf,
      $CustomAttribute1,$CustomAttribute2,$CustomAttribute3,$CustomAttribute4,
      $CustomAttribute5,$CustomAttribute6,$CustomAttribute7,$CustomAttribute8,
      $CustomAttribute9,$CustomAttribute10,$CustomAttribute11,$CustomAttribute12,
      $CustomAttribute13,$CustomAttribute14,$CustomAttribute15)
$error.Clear()
# Build the list of Proxy Email Addresses
$EmailAddresses=New-Object System.Collections.ArrayList
if (![String]::IsNullorEmpty($UserEmail))
   {
	$index=$EmailAddresses.Add("SIP:$($UserEmail)")
	$index=$EmailAddresses.Add("SMTP:$($UserEmail)")
	if (![String]::IsNullorEmpty($UserEmail2))
	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail2)")
	   }
	if (![String]::IsNullorEmpty($UserEmail3))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail3)")
   	   }
	if (![String]::IsNullorEmpty($UserEmail4))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail4)")
   	   }
	if (![String]::IsNullorEmpty($UserEmail5))
  	   {
	   $index=$EmailAddresses.Add("smtp:$($UserEmail5)")
   	   }
	}
# Add Proxy Email Addresses and Custom Attributes    
$error.Clear()
if (![String]::IsNullorEmpty($MailboxPlan))
   {
    $S="MailboxPlan=$($MailboxPlan)"   
    echo $S
    %{Invoke-Command -Session $Script:RS {param ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15,$16,$17,$18) set-mailbox -Identity $1 -EmailAddresses $2 -MailboxPlan $3 -CustomAttribute1 $4 -CustomAttribute2 $5 -CustomAttribute3 $6 -CustomAttribute4 $7 -CustomAttribute5 $8 -CustomAttribute6 $9 -CustomAttribute7 $10 -CustomAttribute8 $11 -CustomAttribute9 $12 -CustomAttribute10 $13 -CustomAttribute11 $14 -CustomAttribute12 $15 -CustomAttribute13 $16 -CustomAttribute14 $17 -CustomAttribute15 $18}-arg $UserID,$EmailAddresses,$MailboxPlan,$CustomAttribute1,$CustomAttribute2,$CustomAttribute3,$CustomAttribute4,$CustomAttribute5,$CustomAttribute6,$CustomAttribute7,$CustomAttribute8,$CustomAttribute9,$CustomAttribute10,$CustomAttribute11,$CustomAttribute12,$CustomAttribute13,$CustomAttribute14,$CustomAttribute15}>$Results
   }
   else
   {
    %{Invoke-Command -Session $Script:RS {param ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15,$16,$17) set-mailbox -Identity $1 -EmailAddresses $2 -CustomAttribute1 $3 -CustomAttribute2 $4 -CustomAttribute3 $5 -CustomAttribute4 $6 -CustomAttribute5 $7 -CustomAttribute6 $8 -CustomAttribute7 $9 -CustomAttribute8 $10 -CustomAttribute9 $11 -CustomAttribute10 $12 -CustomAttribute11 $13 -CustomAttribute12 $14 -CustomAttribute13 $15 -CustomAttribute14 $16 -CustomAttribute15 $17}-arg $UserID,$EmailAddresses,$CustomAttribute1,$CustomAttribute2,$CustomAttribute3,$CustomAttribute4,$CustomAttribute5,$CustomAttribute6,$CustomAttribute7,$CustomAttribute8,$CustomAttribute9,$CustomAttribute10,$CustomAttribute11,$CustomAttribute12,$CustomAttribute13,$CustomAttribute14,$CustomAttribute15}>$Results
   } 	
if ([String]::IsNullorEmpty($error[0]))
	{
	$S="Successfully update mailbox for $($UserDisplayName) ($($UserID)) ($($UserEmail))"
    log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Updating mailbox for $($UserDisplayName) ($($UserID)) ($($UserEmail)). Error = $($error[0])"
    log_message "ERROR" $S
    }
# Add User Misc. Attributes    
# 
$error.Clear()
%{Invoke-Command -Session $Script:RS {param ($id,$title,$displayname,$firstname,$lastname,$initials,$phone,$mobile,$fax,$webpage,$department,$office,$company,$notes) set-user -Identity $id -Title $title -DisplayName $displayname -FirstName $firstname -LastName $lastname -Initials $initials -Phone $phone -MobilePhone $mobile -Fax $fax -WebPage $webpage -Department $department -Office $office -Company $company -Notes $notes}-arg $UserID,$UserTitle,$UserDisplayName,$UserFirstName,$UserLastName,$UserInitials,$UserPhone,$UserMobilePhone,$UserFaxNumber,$UserWebPage,$UserDepartment,$UserOffice,$UserCompany,$UserNotes}>$Results
if ([String]::IsNullorEmpty($error[0]))
	{
	$S="Successfully update user's informations for $($UserDisplayName) ($($UserID)) ($($UserEmail))"
    log_message "OK" $S
	}
	else
	{
	$global:errors_count++
    $S="Updating user's informatios for $($UserDisplayName) ($($UserID)) ($($UserEmail)). Error = $($error[0])"
    log_message "ERROR" $S
    }
# Add membership to distribution groups
$Groups=$MemberOf.Split(",");
ForEach ($Group in $Groups)
        {
        if (![String]::IsNullorEmpty($Group))
           {
           $error.Clear()
           %{Invoke-Command -Session $Script:RS -ErrorAction SilentlyContinue {param ($grp,$memb) add-distributiongroupmember -Identity $grp -Member $memb}-arg $Group,$UserID}
           if ([String]::IsNullorEmpty($error[0]))
   	          {
              $S="Successfully add distribution group ""$($Group)"" for $($UserDisplayName) ($($UserID)) ($($UserEmail))"
              write-host $S
              log_message "OK" $S
              }
           }
        }  
}
###############################################################################
#
###############################################################################
#                                                                             #
#                                    Main                                     #
#                                                                             #
###############################################################################
#
$global:Version="V2.3"
#
# Office 365 params
#
$Office365Url="https://ps.outlook.com/powershell/"
$Office365LoginName="%OFFICE365_ADMIN%"
$Office365LoginPassword="%OFFICE365_PASSWORD%"
#
# Proxy parameters
#
$ProxyAddress="%PROXY_ADDRESS%"
$ProxyUser="%PROXY_USER%"     
$ProxyPassword="%PROXY_PASSWORD%"
#
#
$LogFile=$MyInvocation.MyCommand.Definition 
$LogFile=$LogFile -replace(".ps1",".log")
$global:total_errors_count=0
welcome
check_version
proxy $ProxyAddress $ProxyUser $ProxyPassword
connect $Office365Url $Office365LoginName $Office365LoginPassword
if ($global:total_errors_count -eq 0)
{
### BEGIN_REPETITION ###
$global:errors_count=0
# KoXo Administrator's variables
$UserID="%USER_ID%"
$UserEmail="%USER_EMAIL%"
$UserPassword="%USER_PASSWORD%"
$UserDisplayName="%USER_FIRST_NAME% %USER_LAST_NAME%"
$UserDisplayName=$UserDisplayName.Trim()
$UserTitle="%USER_TITLE%"
$UserFirstName="%USER_FIRST_NAME%"
$UserLastName="%USER_LAST_NAME%"
$UserInitials="%USER_INITIALS%"
$UserPhone="%USER_PHONE%"
$UserMobilePhone="%USER_MOBILE_PHONE%"
$UserFaxNumber="%USER_FAX_NUMBER%"
$UserWebPage="%USER_WEB_PAGE%"
$UserDepartment="%PRIMARY_GROUP% / %SECONDARY_GROUP%"
$UserOffice="%SECONDARY_GROUP%"
$UserCompany="%COMPANY%"
$UserEmail2="%USER_EMAIL[2]%"
$UserEmail3="%USER_EMAIL[3]%"
$UserEmail4="%USER_EMAIL[4]%"
$UserEmail5="%USER_EMAIL[5]%"
$UserNotes="%USER_REMARKS%" -replace ("/","`n")
$MailboxPlan=""
$CustomAttribute1 ="#LOWER{#REMOVE_SPACE{%PRIMARY_GROUP%}}"
$CustomAttribute2 ="#LOWER{#REMOVE_SPACE{%SECONDARY_GROUP_EMAIL%}}"
$CustomAttribute3 =""
$CustomAttribute4 =""
$CustomAttribute5 =""
$CustomAttribute6 =""
$CustomAttribute7 =""
$CustomAttribute8 =""
$CustomAttribute9 =""
$CustomAttribute10=""
$CustomAttribute11=""
$CustomAttribute12=""
$CustomAttribute13=""
$CustomAttribute14=""
$CustomAttribute15=""
$MemberOf="$($CustomAttribute1),$($CustomAttribute2)"
#
log_message "Office 365" "$($UserDisplayName) ($($UserEmail))"
if (user_exists($UserID))
   {
   write-host "User : $($UserDisplayName) ($($UserID)) ($($UserEmail))"
   modify_user $UserID $UserEmail $UserDisplayName $UserTitle $UserFirstName $UserLastName $UserInitials $UserPhone $UserMobilePhone $UserFaxNumber $UserWebPage $UserDepartment $UserOffice $UserCompany $UserEmail2 $UserEmail3 $UserEmail4 $UserEmail5 $UserNotes $MailboxPlan $MemberOf $CustomAttribute1 $CustomAttribute2 $CustomAttribute3 $CustomAttribute4 $CustomAttribute5 $CustomAttribute6 $CustomAttribute7 $CustomAttribute8 $CustomAttribute9 $CustomAttribute10 $CustomAttribute11 $CustomAttribute12 $CustomAttribute13 $CustomAttribute14 $CustomAttribute15 
   }
   else
   {
   $errors_count++
   $S="User ""$($UserID)"" does not exists"
   write-host $S -foregroundcolor "red"
   log_message "ERROR" $S
   }
log_errors_count($UserID)   
$global:total_errors_count=$global:total_errors_count+$global:errors_count
### END_REPETITION ###
close
}
log_total_errors_count
